/*******************************************************************************
 * Copyright (C) 2020, OpenRefine contributors
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package org.openrefine.benchmark;

import java.util.Properties;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Level;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.OutputTimeUnit;
import org.openjdk.jmh.annotations.Param;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.Setup;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;
import org.openjdk.jmh.infra.Blackhole;

import com.google.refine.expr.functions.ToNumber;

public class ToNumberBenchmark {

    static Properties bindings = new Properties();

    @State(Scope.Benchmark)
    public static class ExecutionPlan {

        @Param({"1000", "10000" })
        public int iterations;
        
        public ToNumber f;
        String[] args = new String[1];
        String testData; 
        String testDataInt;
        Random rnd = new Random();
        
        @Setup(Level.Invocation)
        public void setUp() {
            f = new ToNumber();
            testData = Double.toString(rnd.nextDouble() * 10000);
            testDataInt = testData.replace(".", "");
        }
    }

    @Benchmark
    @BenchmarkMode(Mode.AverageTime)
    @OutputTimeUnit(TimeUnit.NANOSECONDS)
	@Warmup(iterations = 3, time = 200, timeUnit = TimeUnit.MILLISECONDS)
	@Measurement(iterations = 5, time = 200, timeUnit = TimeUnit.MILLISECONDS)
    public void toDoubleNew(ExecutionPlan plan, Blackhole blackhole) {
        plan.args[0] = plan.testData;
        blackhole.consume(plan.f.call(bindings, plan.args));
    }

    @Benchmark
    @BenchmarkMode(Mode.AverageTime)
    @OutputTimeUnit(TimeUnit.NANOSECONDS)
	@Warmup(iterations = 3, time = 200, timeUnit = TimeUnit.MILLISECONDS)
	@Measurement(iterations = 5, time = 200, timeUnit = TimeUnit.MILLISECONDS)
	@Fork(1)
    public void toLongNew(ExecutionPlan plan, Blackhole blackhole) {
            plan.args[0] = plan.testDataInt;
            blackhole.consume(plan.f.call(bindings, plan.args));
    }
}

