/*******************************************************************************
 * Copyright (C) 2018, OpenRefine contributors
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.google.refine.expr.functions.strings;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.google.refine.expr.EvalError;
import com.google.refine.grel.ControlFunctionRegistry;
import com.google.refine.grel.Function;

public class Find implements Function {

    @Override
    public Object call(Properties bindings, Object[] args) {
        List<String> allMatches = new ArrayList<String>();
        
        if (args.length == 2) {
            Object s = args[0];
            Object p = args[1];
            
            if (s != null && p != null && p instanceof String) {
                int fromIndex = 0;
                while ((fromIndex = s.toString().indexOf(p.toString(), fromIndex)) != -1 ){
                    allMatches.add(p.toString());
                    fromIndex++;
                }
            }
            
            if (s != null && p != null && p instanceof Pattern) {
                Pattern pattern = (Pattern) p;
                Matcher matcher = pattern.matcher(s.toString());
                
                while (matcher.find()) {
                    allMatches.add(matcher.group());
                } 
            }
            
            return allMatches.toArray(new String[0]);
        }
        return new EvalError(ControlFunctionRegistry.getFunctionName(this) + " expects a string or a regex");
    }
    
    @Override
    public String getDescription() {
        return "Outputs an array of all consecutive substrings inside string s that match the substring or regex pattern p. You can supply a substring by putting it in quotes.";
    }
    
    @Override
    public String getParams() {
        return "string s, substring or regex p";
    }
    
    @Override
    public String getReturns() {
        return "array of strings";
    }
}
